package com.unisys.tibcojboss.mdb;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *
 */
public class SampleMDB implements MessageDrivenBean, MessageListener {
   private MessageDrivenContext ctx = null;
   private QueueConnection conn;
   private QueueSession session;
   /**
    * 
    *
    */
   public SampleMDB() {
      System.out.println("SampleMDB constructor");
   }
   /**
    * 
    */
   public void setMessageDrivenContext(MessageDrivenContext ctx) {
      this.ctx = ctx;
      System.out.println("SampleMDB setMessageDrivenContext() " + ctx);
   }
   /**
    * 
    * @throws EJBException
    */
   public void ejbCreate() throws EJBException {
	   System.out.println("SampleMDB ejbCreate() ");
      try {
         setup();
      } catch(Exception e) {
         throw new EJBException("Problems creating SampleMDB", e);
      }
   }
   /**
    * 
    */
   public void ejbRemove() {
	  System.out.println("SampleMDB ejbRemove() ");
      ctx = null;
      try {
         if( session != null ) session.close();
         if( conn != null ) conn.close();
      } catch(JMSException e){
         e.printStackTrace();
      }
   }

   public void onMessage(Message msg){
      try {
    	 System.out.println("SampleMDB Message type: " + msg.getJMSType());
         TextMessage tm = (TextMessage) msg;
         System.out.println("SampleMDB Message text: " + tm.getText());
         System.out.println("SampleMDB Message ID: " + msg.getJMSMessageID());
         String text = tm.getText() + "processed by: "+hashCode();
         Queue dest = (Queue) msg.getJMSReplyTo();
         sendReply(text, dest);
      } catch(Throwable t){
         t.printStackTrace();
      }
   }
   /**
    * 
    * @throws JMSException
    * @throws NamingException
    */
   private void setup() throws JMSException, NamingException {
       InitialContext iniCtx = new InitialContext();
       Object tmp = iniCtx.lookup("java:comp/env/jms/QCF");
       QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
       conn = qcf.createQueueConnection();
       session = conn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
       conn.start();
   }
   /**
    * 
    * @param text
    * @param dest
    * @throws JMSException
    */
   private void sendReply(String text, Queue dest) throws JMSException {
      System.out.println("SampleMDB sending reply to dest: " + dest);
      QueueSender sender = session.createSender(dest);
      TextMessage tm = session.createTextMessage(text);
      sender.send(tm);
      sender.close();
   }

}
